@@ -3,13 +3,24 @@
  * Author name:  Arkadiusz Firus
  * Create date:  07 Sep 08
  * Purpose:    iconv handles
+ *----------------------------------------------------------------------
+ Changes:
+ * 04 Jan 10, daved@physiol.usyd.edu.au: use path specfied with -P to
+ *	load charmap file(s)
  *--------------------------------------------------------------------*/
 
 #include <stdio.h>
+#include <string.h>
 
 #include "malloc.h"
 #include "my_iconv.h"
 #include "util.h"
+#if 1 /* daved 0.21.1 */
+#include "unicode.h"
+#include "path.h"
+#include <unistd.h>
+#include <stdlib.h>
+#endif
 
 my_iconv_t
 my_iconv_open(const char *tocode, const char *fromcode)
@@ -21,8 +32,34 @@ my_iconv_open(const char *tocode, const char *fromcode)
 
 	if ((cd.desc = iconv_open(tocode, fromcode)) == (iconv_t) -1)
 	{
+#if 1 /* daved 0.21.1 */
+		struct path_dir *path_dir_p;
+		char u_success=0;
+		for(path_dir_p = &topdir; path_dir_p->dir_name; path_dir_p = path_dir_p->next)
+		{
+			char *p;
+			p = path_dir_p->dir_name;
+			if(*(p+strlen(p)-1) != '/')
+				p = concatenate(p, "/");
+			p = concatenate(p, fromcode);
+			path=concatenate(p, ".charmap");
+			if(access(path, F_OK|R_OK))
+				continue;
+			u_success++;
+			break;
+		}
+		if(!u_success)
+		{
+#else
 		path = concatenate(CHARMAP_DIR, fromcode);
+#endif
+#if 1 /* daved 0.21.1 */
+		}
+		if((f = fopen(path, "r")) == NULL)
+			fprintf(stderr, "failed to open charmap file %s\n", path);
+#else
 		f = fopen(path, "r");
+#endif
 
 		if (f != NULL)
 		{
@@ -33,7 +70,6 @@ my_iconv_open(const char *tocode, const char *fromcode)
 			{
 				if (c == '<')
 					cd.char_table[i] = get_unicode_char(f);
-
 				leave_line(f);
 				c = fgetc(f);
 			}
@@ -96,7 +132,7 @@ my_iconv_close(my_iconv_t cd)
 			my_free(cd.char_table[i]);
 		}
 
-		my_free(cd.char_table);
+		my_free((void *)cd.char_table);
 		cd.char_table = NULL;
 	}
 
