@@ -1,4 +1,4 @@
-#define DEFAULT_UNRTF_SEARCH_PATH	"/usr/local/lib/unrtf/outputs/"
+#define DEFAULT_UNRTF_SEARCH_PATH	"/usr/local/lib/unrtf/"
 
 char	*search_path;
 int	n_path_dirs;
